@@ -136,6 +136,21 @@ public:
       return Fbt_null(item, false).is_null();
     }
 
+    /*
+      Check at fix_fields() time if any of the items can return a nullable
+      value on conversion to Fbt.
+    */
+    static bool fix_fields_maybe_null_on_conversion_to_fbt(Item **items,
+                                                           uint count)
+    {
+      for (uint i= 0; i < count; i++)
+      {
+        if (Fbt::fix_fields_maybe_null_on_conversion_to_fbt(items[i]))
+          return true;
+      }
+      return false;
+    }
+
   public:
 
     Fbt(Item *item, bool *error, bool warn= true)
@@ -164,6 +179,10 @@ public:
                                              FbtImpl::max_char_length()+1));
       return false;
     }
+    bool to_bool() const
+    {
+      return !this->only_zero_bytes(m_buffer, FbtImpl::binary_length());
+    }
     int cmp(const Binary_string &other) const
     {
       return FbtImpl::cmp(FbtImpl::to_lex_cstring(), other.to_lex_cstring());
@@ -238,6 +257,10 @@ public:
     {
       return singleton();
     }
+    bool val_bool() override
+    {
+      return m_value.to_bool();
+    }
     longlong val_int() override
     {
       return 0;
@@ -275,8 +298,9 @@ public:
       str->append(tmp);
       str->append('\'');
     }
-    Item *get_copy(THD *thd) override
+    Item *do_get_copy(THD *thd) const override
     { return get_item_copy<Item_literal_fbt>(thd, this); }
+    Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
 
     // Non-overriding methods
     void set_value(const Fbt &value)
@@ -632,7 +656,8 @@ public:
       */
       DBUG_ASSERT(item->type_handler()->type_handler_base_or_self()->
                   is_traditional_scalar_type() ||
-                  item->type_handler() == type_handler());
+                  item->type_handler()->type_collection() ==
+                  type_handler()->type_collection());
       return Data_type_compatibility::OK;
     }
     /**
@@ -647,7 +672,8 @@ public:
       // See the DBUG_ASSERT comment in can_optimize_keypart_ref()
       DBUG_ASSERT(item->type_handler()->type_handler_base_or_self()->
                   is_traditional_scalar_type() ||
-                  item->type_handler() == type_handler());
+                  item->type_handler()->type_collection() ==
+                  type_handler()->type_collection());
       return Data_type_compatibility::OK;
     }
     void hash_not_null(Hasher *hasher) override
@@ -852,8 +878,9 @@ public:
     {
       return Item::save_in_field(field, no_conversions);
     }
-    Item *get_copy(THD *thd) override
+    Item *do_get_copy(THD *thd) const override
     { return get_item_copy<Item_copy_fbt>(thd, this); }
+    Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
   };
 
   class Item_char_typecast_func_handler_fbt_to_binary:
@@ -918,7 +945,7 @@ public:
       str->append(singleton()->name().lex_cstring());
       str->append(')');
     }
-    bool fix_length_and_dec() override
+    bool fix_length_and_dec(THD *thd) override
     {
       Type_std_attributes::operator=(Type_std_attributes_fbt());
       if (Fbt::fix_fields_maybe_null_on_conversion_to_fbt(args[0]))
@@ -953,7 +980,7 @@ public:
       Fbt_null tmp(args[0]);
       return null_value= tmp.is_null() || tmp.to_native(to);
     }
-    Item *get_copy(THD *thd) override
+    Item *do_get_copy(THD *thd) const override
     { return get_item_copy<Item_typecast_fbt>(thd, this); }
   };
 
@@ -963,9 +990,10 @@ public:
   public:
     Item_cache_fbt(THD *thd)
      :Item_cache(thd, singleton()) { }
-    Item *get_copy(THD *thd)
+    Item *do_get_copy(THD *thd) const override
     { return get_item_copy<Item_cache_fbt>(thd, this); }
-    bool cache_value()
+    Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
+    bool cache_value() override
     {
       if (!example)
         return false;
@@ -975,54 +1003,54 @@ public:
                                                    &m_value, type_handler());
       return true;
     }
-    String* val_str(String *to)
+    String* val_str(String *to) override
     {
       if (!has_value())
         return NULL;
       Fbt_null tmp(m_value.ptr(), m_value.length());
       return tmp.is_null() || tmp.to_string(to) ? NULL : to;
     }
-    my_decimal *val_decimal(my_decimal *to)
+    my_decimal *val_decimal(my_decimal *to) override
     {
       if (!has_value())
         return NULL;
       my_decimal_set_zero(to);
       return to;
     }
-    longlong val_int()
+    longlong val_int() override
     {
       if (!has_value())
         return 0;
       return 0;
     }
-    double val_real()
+    double val_real() override
     {
       if (!has_value())
         return 0;
       return 0;
     }
-    longlong val_datetime_packed(THD *thd)
+    longlong val_datetime_packed(THD *) override
     {
       DBUG_ASSERT(0);
       if (!has_value())
         return 0;
       return 0;
     }
-    longlong val_time_packed(THD *thd)
+    longlong val_time_packed(THD *) override
     {
       DBUG_ASSERT(0);
       if (!has_value())
         return 0;
       return 0;
     }
-    bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate)
+    bool get_date(THD *, MYSQL_TIME *ltime, date_mode_t) override
     {
       if (!has_value())
         return true;
       set_zero_time(ltime, MYSQL_TIMESTAMP_TIME);
       return false;
     }
-    bool val_native(THD *thd, Native *to)
+    bool val_native(THD *, Native *to) override
     {
       if (!has_value())
         return true;
@@ -1108,6 +1136,11 @@ public:
     return FbtImpl::max_char_length();
   }
 
+  const Type_handler *type_handler_for_implicit_upgrade() const override
+  {
+    return TypeCollectionImpl::singleton()->
+             type_handler_for_implicit_upgrade(this);
+  }
   const Type_handler *type_handler_for_comparison() const override
   {
     return this;
@@ -1191,10 +1224,10 @@ public:
     return false;
   }
 
-  bool Column_definition_prepare_stage1(THD *thd, MEM_ROOT *mem_root,
+  bool Column_definition_prepare_stage1(THD *, MEM_ROOT *,
                                         Column_definition *def,
-                                        handler *file, ulonglong table_flags,
-                                        const Column_derived_attributes *derived_attr)
+                                        column_definition_type_t,
+                                        const Column_derived_attributes *)
                                         const override
   {
     def->prepare_stage1_simple(&my_charset_numeric);
@@ -1532,6 +1565,16 @@ public:
     Fbt_null na(a), nb(b);
     return !na.is_null() && !nb.is_null() && !na.cmp(nb);
   }
+  bool Item_bool_rowready_func2_fix_length_and_dec(THD *thd,
+                                 Item_bool_rowready_func2 *func) const override
+  {
+    if (Type_handler::Item_bool_rowready_func2_fix_length_and_dec(thd, func))
+      return true;
+    if (!func->maybe_null() &&
+        Fbt::fix_fields_maybe_null_on_conversion_to_fbt(func->arguments(), 2))
+      func->set_maybe_null();
+    return false;
+  }
   bool Item_hybrid_func_fix_attributes(THD *thd, const LEX_CSTRING &name,
                                        Type_handler_hybrid_field_type *h,
                                        Type_all_attributes *attr,
@@ -1713,6 +1756,9 @@ public:
 
   bool Item_func_between_fix_length_and_dec(Item_func_between *func) const override
   {
+    if (!func->maybe_null() &&
+        Fbt::fix_fields_maybe_null_on_conversion_to_fbt(func->arguments(), 3))
+      func->set_maybe_null();
     return false;
   }
   longlong Item_func_between_val_int(Item_func_between *func) const override
@@ -1735,6 +1781,10 @@ public:
                                                     Item_func_in *func)
                                                     const override
   {
+    if (!func->maybe_null() &&
+        Fbt::fix_fields_maybe_null_on_conversion_to_fbt(func->arguments(),
+                                                        func->argument_count()))
+      func->set_maybe_null();
     if (func->compatible_types_scalar_bisection_possible())
     {
       return func->value_list_convert_const_to_int(thd) ||
@@ -1909,6 +1959,12 @@ public:
     return NULL;
   }
 
+  const Type_handler *type_handler_for_implicit_upgrade(
+                                               const Type_handler *from) const
+  {
+    return from;
+  }
+
   static Type_collection_fbt *singleton()
   {
     static Type_collection_fbt tc;
